package com.ruoyi.business.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.business.domain.HwDeviceModeFunction;
import com.ruoyi.business.service.IHwDeviceModeFunctionService;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.page.TableDataInfo;

/**
 * 设备模型功能Controller
 * 
 * @author xins
 * @date 2023-09-05
 */
@RestController
@RequestMapping("/deviceModeFunction")
public class HwDeviceModeFunctionController extends BaseController
{
    @Autowired
    private IHwDeviceModeFunctionService hwDeviceModeFunctionService;

    /**
     * 查询设备模型功能列表
     */
    @RequiresPermissions("business:deviceModeFunction:list")
    @GetMapping("/list")
    public TableDataInfo list(HwDeviceModeFunction hwDeviceModeFunction)
    {
        startPage();
        List<HwDeviceModeFunction> list = hwDeviceModeFunctionService.selectHwDeviceModeFunctionList(hwDeviceModeFunction);
        return getDataTable(list);
    }

    /**
     * 导出设备模型功能列表
     */
    @RequiresPermissions("business:deviceModeFunction:export")
    @Log(title = "设备模型功能", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HwDeviceModeFunction hwDeviceModeFunction)
    {
        List<HwDeviceModeFunction> list = hwDeviceModeFunctionService.selectHwDeviceModeFunctionList(hwDeviceModeFunction);
        ExcelUtil<HwDeviceModeFunction> util = new ExcelUtil<HwDeviceModeFunction>(HwDeviceModeFunction.class);
        util.exportExcel(response, list, "设备模型功能数据");
    }

    /**
     * 获取设备模型功能详细信息
     */
    @RequiresPermissions("business:deviceModeFunction:query")
    @GetMapping(value = "/{modeFunctionId}")
    public AjaxResult getInfo(@PathVariable("modeFunctionId") Long modeFunctionId)
    {
        return success(hwDeviceModeFunctionService.selectHwDeviceModeFunctionByModeFunctionId(modeFunctionId));
    }

    /**
     * 新增设备模型功能
     */
    @RequiresPermissions("business:deviceModeFunction:add")
    @Log(title = "设备模型功能", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HwDeviceModeFunction hwDeviceModeFunction)
    {
        return toAjax(hwDeviceModeFunctionService.insertHwDeviceModeFunction(hwDeviceModeFunction));
    }

    /**
     * 修改设备模型功能
     */
    @RequiresPermissions("business:deviceModeFunction:edit")
    @Log(title = "设备模型功能", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HwDeviceModeFunction hwDeviceModeFunction)
    {
        return toAjax(hwDeviceModeFunctionService.updateHwDeviceModeFunction(hwDeviceModeFunction));
    }

    /**
     * 删除设备模型功能
     */
    @RequiresPermissions("business:deviceModeFunction:remove")
    @Log(title = "设备模型功能", businessType = BusinessType.DELETE)
	@DeleteMapping("/{modeFunctionIds}")
    public AjaxResult remove(@PathVariable Long[] modeFunctionIds)
    {
        return toAjax(hwDeviceModeFunctionService.deleteHwDeviceModeFunctionByModeFunctionIds(modeFunctionIds));
    }
}
