package com.ruoyi.business.service.impl;

import java.util.List;
import com.ruoyi.common.core.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.ArrayList;
import com.ruoyi.common.core.utils.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import com.ruoyi.business.domain.HwDeviceModeFunction;
import com.ruoyi.business.mapper.HwDeviceModeMapper;
import com.ruoyi.business.domain.HwDeviceMode;
import com.ruoyi.business.service.IHwDeviceModeService;

/**
 * 设备模型Service业务层处理
 * 
 * @author xins
 * @date 2023-09-04
 */
@Service
public class HwDeviceModeServiceImpl implements IHwDeviceModeService 
{
    @Autowired
    private HwDeviceModeMapper hwDeviceModeMapper;

    /**
     * 查询设备模型
     * 
     * @param deviceModeId 设备模型主键
     * @return 设备模型
     */
    @Override
    public HwDeviceMode selectHwDeviceModeByDeviceModeId(Long deviceModeId)
    {
        return hwDeviceModeMapper.selectHwDeviceModeByDeviceModeId(deviceModeId);
    }

    /**
     * 查询设备模型列表
     * 
     * @param hwDeviceMode 设备模型
     * @return 设备模型
     */
    @Override
    public List<HwDeviceMode> selectHwDeviceModeList(HwDeviceMode hwDeviceMode)
    {
        return hwDeviceModeMapper.selectHwDeviceModeList(hwDeviceMode);
    }

    /**
     * 新增设备模型
     * 
     * @param hwDeviceMode 设备模型
     * @return 结果
     */
    @Transactional
    @Override
    public int insertHwDeviceMode(HwDeviceMode hwDeviceMode)
    {
        hwDeviceMode.setCreateTime(DateUtils.getNowDate());
        int rows = hwDeviceModeMapper.insertHwDeviceMode(hwDeviceMode);
        insertHwDeviceModeFunction(hwDeviceMode);
        return rows;
    }

    /**
     * 修改设备模型
     * 
     * @param hwDeviceMode 设备模型
     * @return 结果
     */
    @Transactional
    @Override
    public int updateHwDeviceMode(HwDeviceMode hwDeviceMode)
    {
        hwDeviceMode.setUpdateTime(DateUtils.getNowDate());
        hwDeviceModeMapper.deleteHwDeviceModeFunctionByDeviceModeId(hwDeviceMode.getDeviceModeId());
        insertHwDeviceModeFunction(hwDeviceMode);
        return hwDeviceModeMapper.updateHwDeviceMode(hwDeviceMode);
    }

    /**
     * 批量删除设备模型
     * 
     * @param deviceModeIds 需要删除的设备模型主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteHwDeviceModeByDeviceModeIds(Long[] deviceModeIds)
    {
        hwDeviceModeMapper.deleteHwDeviceModeFunctionByDeviceModeIds(deviceModeIds);
        return hwDeviceModeMapper.deleteHwDeviceModeByDeviceModeIds(deviceModeIds);
    }

    /**
     * 删除设备模型信息
     * 
     * @param deviceModeId 设备模型主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteHwDeviceModeByDeviceModeId(Long deviceModeId)
    {
        hwDeviceModeMapper.deleteHwDeviceModeFunctionByDeviceModeId(deviceModeId);
        return hwDeviceModeMapper.deleteHwDeviceModeByDeviceModeId(deviceModeId);
    }

    /**
     * 新增设备模型功能信息
     * 
     * @param hwDeviceMode 设备模型对象
     */
    public void insertHwDeviceModeFunction(HwDeviceMode hwDeviceMode)
    {
        List<HwDeviceModeFunction> hwDeviceModeFunctionList = hwDeviceMode.getHwDeviceModeFunctionList();
        Long deviceModeId = hwDeviceMode.getDeviceModeId();
        if (StringUtils.isNotNull(hwDeviceModeFunctionList))
        {
            List<HwDeviceModeFunction> list = new ArrayList<HwDeviceModeFunction>();
            for (HwDeviceModeFunction hwDeviceModeFunction : hwDeviceModeFunctionList)
            {
                hwDeviceModeFunction.setDeviceModeId(deviceModeId);
                list.add(hwDeviceModeFunction);
            }
            if (list.size() > 0)
            {
                hwDeviceModeMapper.batchHwDeviceModeFunction(list);
            }
        }
    }
}
